#ifndef __SYMBIANUNIT_H__
#define __SYMBIANUNIT_H__


#include <e32std.h>
#include <e32base.h>

const TInt KSymbianUnitUidValue = 0x10000518;
const TUid KSymbianUnitUid = {KSymbianUnitUidValue};

class CTestResult;

class CTest : public CBase
	{
public:
	IMPORT_C void SetName(const TDesC& aName);
	IMPORT_C const TDesC& Name() const;
	virtual void RunL(CTestResult& aResult) = 0;
private:
	TBuf<255> iName;
	};

class MTestListener
	{
public:
	IMPORT_C virtual void StartTest(const CTest& aTest) = 0;
	IMPORT_C virtual void EndTest(const CTest& aTest) = 0;
	IMPORT_C virtual void StartSuite(const CTest& aTest) = 0;
	IMPORT_C virtual void EndSuite(const CTest& aTest) = 0;
	};

class CTestFailure : public CBase
	{
public:
	IMPORT_C static CTestFailure* NewL(const TDesC& aMessage, const TDesC& aFileName, const TUint aLineNumber);
	IMPORT_C ~CTestFailure();
	IMPORT_C const TDesC& Message() const;
	IMPORT_C const TDesC& FileName() const;
	IMPORT_C const TUint LineNumber() const;
private:
	CTestFailure();
	void ConstructL(const TDesC& aMessage, const TDesC& aFileName, const TUint aLineNumber);
private:
	HBufC* iMessage;
	HBufC* iFileName;
	TUint iLineNumber;
	};

class CTestResult : public CBase
	{
public:
	IMPORT_C static CTestResult* NewLC();
	IMPORT_C static CTestResult* NewL();
	IMPORT_C ~CTestResult();
	IMPORT_C void AddListenerL(MTestListener& aListener);
	IMPORT_C void StartTest(const CTest& aTest);
	IMPORT_C void EndTest(const CTest& aTest);
	IMPORT_C void StartSuite(const CTest& aTest);
	IMPORT_C void EndSuite(const CTest& aTest);
	IMPORT_C void AddFailureL(const CTestFailure* aFailure);
	IMPORT_C TBool HasFailure() const;
	IMPORT_C TUint RunCount() const;
	IMPORT_C TUint FailCount() const;
	IMPORT_C TUint PassCount() const;
	IMPORT_C const CTestFailure& Failure(TUint index) const;
private:
	CTestResult();
	void ConstructL();
private:
	CArrayPtrFlat<const CTestFailure>* iFailures;
	TUint iRun;
	MTestListener* iListener;
	};

class CTestSuite : public CTest
	{
public:
	IMPORT_C static CTestSuite* NewLC();
	IMPORT_C static CTestSuite* NewL();
	IMPORT_C ~CTestSuite();
	IMPORT_C void RunL(CTestResult& aResult);
	IMPORT_C void AddTestL(CTest* aTest);
private:
	CTestSuite();
	void ConstructL();
private:
	CArrayPtrFlat<CTest>* iTests;
	};

template <class Fixture>
class CTestMethod : public CTest
	{
private:
	typedef void (Fixture::*TestMethod)();
public:
	static CTestMethod<Fixture>* NewL(const TDesC& aName, Fixture& aFixture, TestMethod aTest)
		{
		CTestMethod<Fixture>* self = new (ELeave) CTestMethod<Fixture>(aFixture, aTest);
		self->SetName(aName);
		return self;
		}

	void RunL(CTestResult& aResult)
		{
		TInt leaveCode;
		aResult.StartTest(*this);
		iFixture.__iResult = &aResult;
		TRAP(leaveCode, iFixture.SetUpL(); (iFixture.*iTest)());
		iFixture.TearDown();
		aResult.EndTest(*this);
		}
private:
	CTestMethod(Fixture& aFixture, TestMethod aTest)
		: iFixture(aFixture), iTest(aTest)
		{
		}
private:
	Fixture& iFixture;
	TestMethod iTest;
	};

class CTestCase : public CTest
	{
public:
	IMPORT_C void RunL(CTestResult& aResult);
	IMPORT_C virtual void SetUpL();
	IMPORT_C virtual void TearDown();
	virtual CTestSuite* SuiteLC() = 0;
protected:
	EXPORT_C void _Failed(const char* aMessage, const char* aFileName, const TUint aLineNumber);
public:
	CTestResult* __iResult;
	};

const TInt KErrAssertionFailed = -20999;

#define SUNIT_FAIL(message) this->_Failed(message, __FILE__, __LINE__);

#define SUNIT_ASSERT(condition) if (!(condition)) {\
	this->_Failed(#condition, __FILE__, __LINE__);\
}

#define SUNIT_TESTCASE_TABLE_DECLARE CTestSuite* SuiteLC()
#define SUNIT_TESTCASE_TABLE_BEGIN(aFixtureClass) typedef CTestMethod<aFixtureClass> aFixtureClass::__TEST_METHOD;\
CTestSuite* aFixtureClass::SuiteLC() { CTestSuite* suite = CTestSuite::NewLC(); suite->SetName(TPtrC(L ## #aFixtureClass));
#define SUNIT_TESTCASE_TABLE_END return suite; }
#define SUNIT_TESTCASE(aTestMethod) suite->AddTestL(__TEST_METHOD::NewL(TPtrC(L ## #aTestMethod), *this, this->aTestMethod));


#endif
