#include "ConsoleTestRunner.h"
#ifdef EKA2
#include <e32debug.h>
#else
#include <e32svr.h>
#endif //EKA2
#include <bacline.h>

CConsoleTestRunner* CConsoleTestRunner::NewLC(CConsoleBase& aConsole)
	{
	CConsoleTestRunner* self = new (ELeave)CConsoleTestRunner(aConsole);
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

CConsoleTestRunner::CConsoleTestRunner(CConsoleBase& aConsole)
	: iConsole(aConsole), iSuiteDepth(0)
	{
	}

void CConsoleTestRunner::ConstructL()
	{
	_LIT(KUsage, "ConsoleTestRunner.exe <FileName>\n");
	_LIT(KCantLoadDll, "Cannot load %S.\n");
	_LIT(KIncorrectUid, "DLL has incorrect UID.\n");
	_LIT(KLoadingDll, "Loading test DLL %S.\n");
	_LIT(KDefaultTestDll, "SymbianUnitTest.dll");
	
	TFileName dllFileName;
	CCommandLineArguments* args = CCommandLineArguments::NewLC();
	if (args->Count() != 2)
		{
		iConsole.Printf(KUsage);
		dllFileName.Copy(KDefaultTestDll);
		}
	else
		{
		dllFileName.Copy(args->Arg(1));
		}
	CleanupStack::PopAndDestroy(args);

	iConsole.Printf(KLoadingDll, &dllFileName);
	if (KErrNone != iLibrary.Load(dllFileName))
		{
		iConsole.Printf(KCantLoadDll, &dllFileName);
		User::Leave(KErrGeneral);
		}
	if (iLibrary.Type()[1] != KSymbianUnitUid)
		{
		iConsole.Printf(KIncorrectUid);
		User::Leave(KErrGeneral);
		}
	
	TLibraryFunction entry = iLibrary.Lookup(1);
	iTest = (CTest*) entry();
	}

CConsoleTestRunner::~CConsoleTestRunner()
	{
	delete iTest;
	iLibrary.Close();
	}

void CConsoleTestRunner::RunTestsL(CTest& aTest)
	{
	_LIT(KFailureFormat, "Failure at %S(%d): %S\n");
	_LIT(KResultFormat, "Passed: %d, Failed: %d\n");
	_LIT(KOkMessage, "Ok.\n");
	_LIT(KFailMessage, "Fail.\n");

	CTestResult* result = CTestResult::NewLC();
	result->AddListenerL(*this);
	aTest.RunL(*result);
	for (TInt i = 0; i < result->FailCount(); i++)
		{
		const CTestFailure& failure = result->Failure(i);
		iConsole.Printf(KFailureFormat, &failure.FileName(), failure.LineNumber(), &failure.Message());
		}
	iConsole.Printf(KResultFormat, result->PassCount(), result->FailCount());
	if (result->HasFailure())
		{
		iConsole.Printf(KFailMessage);
		}
	else
		{
		iConsole.Printf(KOkMessage);
		}
	CleanupStack::PopAndDestroy(result);
	}

void CConsoleTestRunner::ExecuteL()
	{
	RunTestsL(*iTest);
	}

void CConsoleTestRunner::LogEvent(const TUint aDepth, const TDesC& aLog)
	{
	_LIT(KIndent, "  ");
	TBuf<1024> message;
	for (TUint i = 0; i < aDepth; i++)
		{
		message.Append(KIndent);
		}
	message.Append(aLog);
	iConsole.Write(message);
	RDebug::RawPrint(message);
	}

void CConsoleTestRunner::StartTest(const CTest& aTest)
	{
	_LIT(KMessageFormat, "Test %S started.\n");
	TBuf<128> message;

	message.Format(KMessageFormat, &aTest.Name());
	LogEvent(iSuiteDepth, message);
	}

void CConsoleTestRunner::EndTest(const CTest& aTest)
	{
	_LIT(KMessageFormat, "Test %S ended.\n");
	TBuf<128> message;

	message.Format(KMessageFormat, &aTest.Name());
	LogEvent(iSuiteDepth, message);
	}

void CConsoleTestRunner::StartSuite(const CTest& aTest)
	{
	_LIT(KMessageFormat, "Test suite %S started.\n");
	TBuf<128> message;

	message.Format(KMessageFormat, &aTest.Name());
	LogEvent(iSuiteDepth, message);
	iSuiteDepth++;
	}

void CConsoleTestRunner::EndSuite(const CTest& aTest)
	{
	_LIT(KMessageFormat, "Test suite %S ended.\n");
	TBuf<128> message;

	iSuiteDepth--;
	message.Format(KMessageFormat, &aTest.Name());
	LogEvent(iSuiteDepth, message);
	}


_LIT(KConsoleTitle, "SymbianUnit Test Runner");
_LIT(KLeaveErrorFormat, "Error: leave code=%d\n");
_LIT(KPressAnyKeyMessage, "[Press any key]\n");

CConsoleBase* console;

LOCAL_C void ExecuteTestRunner()
	{
	CConsoleTestRunner* runner = CConsoleTestRunner::NewLC(*console);
	runner->ExecuteL();
	CleanupStack::PopAndDestroy(runner);
	}

GLDEF_C TInt E32Main()
	{
	__UHEAP_MARK;
	CTrapCleanup* cleanupstack = CTrapCleanup::New();

	console = Console::NewL(KConsoleTitle, TSize(KConsFullScreen, KConsFullScreen));
	TRAPD(leaveCode, ExecuteTestRunner());
	if (leaveCode)
		console->Printf(KLeaveErrorFormat, leaveCode);
	console->Printf(KPressAnyKeyMessage);
	console->Getch();
	delete console;

	delete cleanupstack; 
	__UHEAP_MARKEND;
	return 0;
	}